rm(list=ls())
graphics.off()


#### Data Simulation ####
n_rv <- 10                    # Number of covariates
prob_rvar <- seq(0.3,0.7,0.1) # Parameters for the Bernoulli r.v.
N <- 1000                     # Sample size

### Generate Standard Normal independent r.v.
library(mvtnorm)
set.seed(70522)
Norm_RV <- rmvnorm(n=N, mean=rep(0,n_rv/2), sigma=diag(n_rv/2)) 
### Generate Bernoulli independent r.v.
library(MultiRNG)
set.seed(70522)
Ber_RV <- draw.correlated.binary(no.row=N, d=n_rv/2, prop.vec=prob_rvar,
                                 corr.mat=diag(n_rv/2))

cov <- as.data.frame(cbind(Norm_RV, Ber_RV))
colnames(cov) <- paste0("X", 1:n_rv)

### Definition of the partitions
node1 <- cov[,"X6"]==1
node2 <- cov[,"X6"]==0 & cov[,"X1"]>=0 
node3 <- cov[,"X6"]==0 & cov[,"X1"]<0

maxtime <- 3                    #Length of the follow-up
tvec <- seq(1,maxtime,0.1)

simdata <- as.data.frame(matrix(data=NA, nrow=N, ncol=2))
colnames(simdata) <- c("eventtime","status")
library(simsurv)
# Node1
simdata[node1,1:2] <- simsurv(dist="exponential", lambda=2, maxt=maxtime,
                              x=as.data.frame(1:sum(node1)), seed=7052022)[,2:3]
# Node2
simdata[node2,1:2] <- simsurv(dist="exponential", lambda=0.9, maxt=maxtime,
                              x=as.data.frame(1:sum(node2)), seed=7052022)[,2:3]
# Node3
simdata[node3,1:2] <- simsurv(dist="exponential", lambda=0.1, maxt=maxtime,
                              x=as.data.frame(1:sum(node3)), seed=7052022)[,2:3]

data <- cbind(simdata,cov) # Dataset with survival data and covariates
data[,8:12] <- lapply(data[,8:12], as.factor)

### Training and testing sets
set.seed(70522)
# To take a similar percentage of events in the training and test sets:
library(caret)
trainset <- createDataPartition(y=data$status, p=0.5, list=F)
data_train <- data[trainset,]
data_test  <- data[-trainset,]


#### Model Fitting ####
### Relative Risk Trees (RRTs)
library(survival)
library(rpart)
RRT <- rpart(formula=Surv(eventtime, status)~., data=data_train, 
             control=rpart.control(usesurrogate=2, minsplit=20, xval=10))
# Pruning RRT
minerr <- which.min(RRT$cptable[,"xerror"])
bestcp <- RRT$cptable[minerr,"CP"]
pruned_RRT <- prune(tree=RRT, cp=bestcp)
# Plotting RRT
library(rpart.plot) 
rpart.plot(x=pruned_RRT, type=5, tweak=1.0, gap=0.02, branch.lty=2)
# An alternative way of plotting RRT
library(partykit)
plot(as.party(pruned_RRT))

### Conditional Inference Trees with ctree (CITs)
library(partykit)
CIT <- ctree(formula=Surv(eventtime, status)~., data=data_train, 
             control=ctree_control(minsplit=20))
# Plotting CIT
plot(CIT)

### Conditional Inference Trees with SurvCART (SCTs)
# Data pre-processing
data_train_SCT <- data_train
id <- 1:nrow(data_train_SCT)
data_train_SCT <- cbind(data_train_SCT,id)
data_train_SCT[,paste0("X",6:10)] <- 
  lapply(data_train_SCT[,paste0("X",6:10)], as.numeric)
data_train_SCT[, paste0("X",6:10)] <- data_train_SCT[, paste0("X",6:10)]-1

# Model Fitting
library(LongCART)
SCT <- SurvCART(data=data_train_SCT, patid="id", timevar="eventtime",
                censorvar="status", gvars=paste0("X",1:10), 
                tgvars=c(1,1,1,1,1,0,0,0,0,0), time.dist="exponential", 
                cens.dist="NA", minsplit=20)

# Plotting SCT
par(xpd=TRUE)
plot(SCT, compress=TRUE)
text(SCT, use.n=TRUE)
# Plotting Kaplan-Meier curves estimated in the terminal nodes
KMPlot.SurvCART(x=SCT, type=1)


#### Performance Evaluation ####

### Harrel C 

# Overall concordance
# Risk predictions for RRT
pred_risk_RRT <- predict(object=pruned_RRT, newdata=data_test)
head(pred_risk_RRT)
# Survival outcome in the test set		
ySurv_test <- Surv(data_test$eventtime, data_test$status)

library(Hmisc)
(C_ind_RRT <- rcorr.cens(x= -pred_risk_RRT, S=ySurv_test))

# Survival predictions for CIT
pred_surv_CIT <- predict(object=CIT, newdata=data_test, type="response")
head(pred_surv_CIT)
# Replacement of Inf values with a value greater than all the others:
# Replacement of Inf values with a value greater than all the others:
pred_surv_CIT[which(pred_surv_CIT==Inf)] <- 
  max(pred_surv_CIT[-which(pred_surv_CIT==Inf)])+3
(C_ind_CIT <- rcorr.cens(x=pred_surv_CIT, S=ySurv_test))

# Alternative: SurvMetrics package
library(SurvMetrics)
(C_ind_RRT <- Cindex(object=ySurv_test, predicted= -pred_risk_RRT))
(C_ind_CIT <- Cindex(object=ySurv_test, predicted=pred_surv_CIT))

### Concordance at different timepoints - Truncated C-index

# Estimation of survival predictions at different timepoints
library(pec)
RRT_pec <- pecRpart(formula=Surv(eventtime, status)~., data=data_train,
                    cp=bestcp) 
# Survival predictions for RRT	
pred_RRT_pec <- predictSurvProb(object=RRT_pec, newdata=data_test,
                                times=tvec)
colnames(pred_RRT_pec) <- paste0("time=", tvec)
head(pred_RRT_pec)

CIT_pec <- pecCtree(formula=Surv(eventtime, status)~., data=data_train)
# Survival predictions for CIT
pred_CIT_pec <- predictSurvProb(object=CIT_pec, newdata=data_test,
                                times=tvec)
colnames(pred_CIT_pec) <- paste0("time=", tvec)
head(pred_CIT_pec)

C_ind_RRT_vec <- C_ind_RRT_vec1 <- rep(NA, length(tvec))
names(C_ind_RRT_vec) <- names(C_ind_RRT_vec1) <- tvec
for(i in 1:length(tvec))
   {
    eventtime <- data_test$eventtime; status <- data_test$status
    status[eventtime>tvec[i]] <- 0
    eventtime[eventtime>tvec[i]] <- tvec[i]
    ySurv <- Surv(eventtime, status)
    C_ind_RRT_vec[i] <- rcorr.cens(x=pred_RRT_pec[,i], S=ySurv)
    C_ind_RRT_vec1[i] <- Cindex(object=ySurv, predicted=pred_RRT_pec[,i])
   }

C_ind_CIT_vec <- C_ind_CIT_vec1 <- rep(NA, length(tvec))
names(C_ind_CIT_vec) <- names(C_ind_CIT_vec1) <- tvec
for(i in 1:length(tvec))
  {
   eventtime <- data_test$eventtime; status <- data_test$status
   status[eventtime>tvec[i]] <- 0
   eventtime[eventtime>tvec[i]] <- tvec[i]
   ySurv <- Surv(eventtime, status)
   C_ind_CIT_vec[i] <- rcorr.cens(x=pred_CIT_pec[,i], S=ySurv)
   C_ind_CIT_vec1[i] <- Cindex(object=ySurv, predicted=pred_CIT_pec[,i])
  }

### Uno C 

### Survival outcome in the training set		
ySurv_train <- Surv(data_train$eventtime, data_train$status)

library(survAUC)
(UnoC_RRT <- UnoC(Surv.rsp=ySurv_train, Surv.rsp.new=ySurv_test,
                 lpnew=pred_risk_RRT))

(UnoC_CIT <- UnoC(Surv.rsp=ySurv_train, Surv.rsp.new=ySurv_test,
                 lpnew= -pred_surv_CIT))

### Concordance at different timepoints with pec - Truncated C-index 
library(pec)
(TruncC_RRT_CIT <- cindex(object=list("RRT"=RRT_pec, "CIT"=CIT_pec),
                          formula=Surv(eventtime, status)~., data=data_test,
                          eval.times=tvec))

### Time-dependent AUC
library(riskRegression)

(AUC_Score_RRT <- Score(object=list("RRT"=pruned_RRT), metrics="AUC",
                       formula=Surv(eventtime, status)~X1+X2+X3+X4+X5+X6+X7+X8+
                       X9+X10, data=data_test, cens.model="km", 
                       times=tvec))

plot(x=tvec, y=AUC_Score_RRT$AUC$score$AUC, type="l", xlab="Times",
     ylab="Time-dependent AUC", main="RRT")

# Due to the incompatibility of riskRegression with the package partykit 
# Fit CIT with the old package party
CIT_party <- party::ctree(Surv(eventtime, status)~., data=data_train)
# Moreover, internal error: instead of extracting a measure of risk, the
# function extracts for each observation a list of elements. To fix it:

predictRisk.BinaryTree <- function (object, newdata, ...) 
  {
   requireNamespace("party")
   treeresponse <- party::treeresponse
   sapply(treeresponse(object, newdata = newdata), 
         function(x) sum(x$n.event)/x$n)
  }
assignInNamespace("predictRisk.BinaryTree", predictRisk.BinaryTree, 
                  pos="package:riskRegression")

(AUC_Score_CIT <- Score(object=list("CIT_party"=CIT_party), metrics="AUC",
                       formula=Surv(eventtime, status)~X1+X2+X3+X4+X5+X6+X7+X8+
                         X9+X10, data=data_test, cens.model="km", 
                       times=tvec))
plot(x=tvec, y=AUC_Score_CIT$AUC$score$AUC, type="l", xlab="Times",
     ylab="Time-dependent AUC", main="CIT")

# Alternative: survAUC package -- Attention to this function (see Suppl File 1)
library(survAUC)
(AUC_hc_RRT <- AUC.hc(Surv.rsp=ySurv_train, Surv.rsp.new=ySurv_test,
                     lpnew=pred_risk_RRT, times=tvec))

(AUC_hc_CIT <- AUC.hc(Surv.rsp=ySurv_train, Surv.rsp.new=ySurv_test, 
                     lpnew= -pred_surv_CIT, times=tvec))

### Brier score and integrated Brier score
library(SurvMetrics)

# BS at time t=1 for RRT
(brier_RRT_1 <- Brier(object=ySurv_test, pre_sp=pred_RRT_pec[,1], t_star=1))

# BS at time t=1 for CIT
(brier_CIT_1 <- Brier(object=ySurv_test, pre_sp=pred_CIT_pec[,1], t_star=1))                              

# IBS on the interval [1,3] for RRT
(ibs_RRT <- IBS(object=ySurv_test, sp_matrix=pred_RRT_pec, IBSrange=tvec))                                  

# IBS on the interval [1,3] for CIT
(ibs_CIT <- IBS(object=ySurv_test, sp_matrix=pred_CIT_pec, IBSrange=tvec))

# Alternative: pec package
library(pec)
# BS at time t=1 for RRT and CIT
(bs_1 <- pec(object=list("RRT"=RRT_pec, "CIT"=CIT_pec), data=data_test,
            formula=Surv(eventtime, status)~., times=1, exact=F, 
            cens.model="marginal")) 

# IBS on the interval [1,3] for RRT
(ibs_rrt <- pec(object=list("RRT"=RRT_pec), data=data_test,
               formula=Surv(eventtime,status)~., times=tvec,
               cens.model="marginal")) 

(ibs_cit <- pec(object=list("CIT"=CIT_pec), data=data_test, 
               formula=Surv(eventtime,status)~., times=tvec,
               cens.model="marginal"))

